import 'package:plugin_platform_interface/plugin_platform_interface.dart';

import 'vnote_device_plugin_method_channel.dart';

abstract class VnoteDevicePluginPlatform extends PlatformInterface {
  /// Constructs a VnoteDevicePluginPlatform.
  VnoteDevicePluginPlatform() : super(token: _token);

  static final Object _token = Object();

  static VnoteDevicePluginPlatform _instance = MethodChannelVnoteDevicePlugin();

  /// The default instance of [VnoteDevicePluginPlatform] to use.
  ///
  /// Defaults to [MethodChannelVnoteDevicePlugin].
  static VnoteDevicePluginPlatform get instance => _instance;

  /// Platform-specific implementations should set this with their own
  /// platform-specific class that extends [VnoteDevicePluginPlatform] when
  /// they register themselves.
  static set instance(VnoteDevicePluginPlatform instance) {
    PlatformInterface.verifyToken(instance, _token);
    _instance = instance;
  }

  Future<String?> getPlatformVersion() {
    throw UnimplementedError('platformVersion() has not been implemented.');
  }

  Future<void> callAction(String action, Map<String, dynamic> data);

  Future<void> init([bool isDebug = false]);

  /// 设置是否开启日志
  Future<void> setLogEnable(bool enable);

  /// 日志是否开启
  bool get isLogEnable;
}
